@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """The jsonpickle.tags module provides the custom tags
 used for pickling and unpickling Python objects.
 
@@ -8,10 +6,12 @@ created by the Pickler class.  The Unpickler uses
 these custom key names to identify dictionaries
 that need to be specially handled.
 """
-from jsonpickle.compat import set
+from __future__ import absolute_import, division, unicode_literals
 
 
 BYTES = 'py/bytes'
+B64 = 'py/b64'
+B85 = 'py/b85'
 FUNCTION = 'py/function'
 ID = 'py/id'
 INITARGS = 'py/initargs'
@@ -31,7 +31,7 @@ TUPLE = 'py/tuple'
 TYPE = 'py/type'
 
 # All reserved tag names
-RESERVED = set([
+RESERVED = {
     BYTES,
     FUNCTION,
     ID,
@@ -49,4 +49,4 @@ RESERVED = set([
     STATE,
     TUPLE,
     TYPE,
-])
+}
