@@ -34,6 +34,7 @@ class UTC(datetime.tzinfo):
     def dst(self, dt):
         return datetime.timedelta()
 
+
 utc = UTC()
 
 
@@ -80,16 +81,10 @@ class PersistantVariables(object):
         self._data = {}
 
     def __getitem__(self, key):
-        if key not in self._data:
-            self._data[key] = TimestampedVariable(None)
-
-        return self._data[key]
+        return self._data.setdefault(key, TimestampedVariable(None))
 
     def __setitem__(self, key, value):
-        if key not in self._data:
-            self._data[key] = TimestampedVariable(value)
-
-        return self._data[key]
+        return self._data.setdefault(key, TimestampedVariable(value))
 
     def __repr__(self):
         return str(self._data)
@@ -126,7 +121,7 @@ class DateTimeSimpleTestCase(unittest.TestCase):
         """
         pickled = jsonpickle.encode(obj)
         unpickled = jsonpickle.decode(pickled)
-        self.assertEquals(obj, unpickled)
+        self.assertEqual(obj, unpickled)
 
     def test_datetime(self):
         """
@@ -259,5 +254,6 @@ def suite():
     suite.addTest(unittest.makeSuite(DateTimeInnerReferenceTestCase))
     return suite
 
+
 if __name__ == '__main__':
     unittest.main(defaultTest='suite')
