@@ -1,12 +1,10 @@
-# -*- coding: utf-8 -*-
-#
 # Copyright (C) 2008 John Paulett (john -at- paulett.org)
-# Copyright (C) 2009, 2011, 2013 David Aguilar and contributors
+# Copyright (C) 2009-2018 David Aguilar (davvid -at- gmail.com)
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.
-
+from __future__ import absolute_import, division, unicode_literals
 import doctest
 import os
 import unittest
@@ -16,10 +14,8 @@ import jsonpickle
 import jsonpickle.backend
 import jsonpickle.handlers
 
-from jsonpickle import tags, util
-from jsonpickle.compat import unicode
-from jsonpickle.compat import unichr
-from jsonpickle.compat import PY32, PY3
+from jsonpickle import compat, tags, util
+from jsonpickle.compat import PY2, PY3
 
 from helper import SkippableTest
 
@@ -83,24 +79,83 @@ class ThingWithProps(object):
         return self.identity == other.identity
 
 
+class UserDict(dict):
+    """A user class that inherits from :class:`dict`"""
+
+    def __init__(self, **kwargs):
+        dict.__init__(self, **kwargs)
+        self.valid = False
+
+
+class Outer(object):
+    class Middle(object):
+        class Inner(object):
+            pass
+
+
 class PicklingTestCase(unittest.TestCase):
 
     def setUp(self):
         self.pickler = jsonpickle.pickler.Pickler()
         self.unpickler = jsonpickle.unpickler.Unpickler()
+        self.b85_pickler = jsonpickle.pickler.Pickler(use_base85=True)
 
     def tearDown(self):
         self.pickler.reset()
         self.unpickler.reset()
 
+    @unittest.skipIf(not PY2, 'Python 2-specific base85 test')
+    def test_base85_always_false_on_py2(self):
+        self.assertFalse(self.b85_pickler.use_base85)
+
+    @unittest.skipIf(PY2, 'Base85 not supported on Python 2')
+    def test_base85_override_py3(self):
+        """Ensure the Python 2 check still lets us set use_base85 on Python 3"""
+        self.assertTrue(self.b85_pickler.use_base85)
+
+    @unittest.skipIf(PY2, 'Base85 not supported on Python 2')
+    def test_bytes_default_base85(self):
+        data = os.urandom(16)
+        encoded = util.b85encode(data)
+        self.assertEqual({tags.B85: encoded}, self.b85_pickler.flatten(data))
+
+    @unittest.skipIf(PY2, 'Base85 not supported on Python 2')
+    def test_py3_bytes_base64_default(self):
+        data = os.urandom(16)
+        encoded = util.b64encode(data)
+        self.assertEqual({tags.B64: encoded}, self.pickler.flatten(data))
+
+    @unittest.skipIf(not PY2, 'Python 2-specific base64 test')
+    def test_py2_default_base64(self):
+        data = os.urandom(16)
+        encoded = util.b64encode(data)
+        self.assertEqual({tags.B64: encoded}, self.pickler.flatten(data))
+
+    @unittest.skipIf(PY2, 'Base85 not supported on Python 2')
+    def test_decode_base85(self):
+        pickled = {tags.B85: 'P{Y4;Xv4O{u^=-c'}
+        expected = u'P\u00ffth\u00f6\u00f1 3!'.encode('utf-8')
+        self.assertEqual(expected, self.unpickler.restore(pickled))
+
+    @unittest.skipIf(PY2, 'Base85 not supported on Python 2')
+    def test_base85_still_handles_base64(self):
+        pickled = {tags.B64: 'UMO/dGjDtsOxIDMh'}
+        expected = u'P\u00ffth\u00f6\u00f1 3!'.encode('utf-8')
+        self.assertEqual(expected, self.unpickler.restore(pickled))
+
+    @unittest.skipIf(not PY2, 'Python 2-specific base85 test')
+    def test_base85_crashes_py2(self):
+        with self.assertRaises(NotImplementedError):
+            self.unpickler.restore({tags.B85: 'P{Y4;Xv4O{u^=-c'})
+
     def test_string(self):
         self.assertEqual('a string', self.pickler.flatten('a string'))
         self.assertEqual('a string', self.unpickler.restore('a string'))
 
     def test_unicode(self):
-        self.assertEqual(unicode('a string'),
+        self.assertEqual('a string',
                          self.pickler.flatten('a string'))
-        self.assertEqual(unicode('a string'),
+        self.assertEqual('a string',
                          self.unpickler.restore('a string'))
 
     def test_int(self):
@@ -176,7 +231,7 @@ class PicklingTestCase(unittest.TestCase):
         self.assertEqual(data, newdata)
 
     def test_set_roundtrip(self):
-        data = set([1, 2, 3])
+        data = {1, 2, 3}
         newdata = jsonpickle.decode(jsonpickle.encode(data))
         self.assertEqual(data, newdata)
 
@@ -269,7 +324,7 @@ class PicklingTestCase(unittest.TestCase):
     def test_set_not_unpicklable(self):
         self.pickler.unpicklable = False
 
-        flattened = self.pickler.flatten(set(['one', 2, 3]))
+        flattened = self.pickler.flatten({'one', 2, 3})
         self.assertTrue('one' in flattened)
         self.assertTrue(2 in flattened)
         self.assertTrue(3 in flattened)
@@ -311,7 +366,7 @@ class PicklingTestCase(unittest.TestCase):
 
         flattened = self.pickler.flatten(obj)
         self.assertEqual(flattened['typeref'],
-                         {tags.TYPE: '__builtin__.object'})
+                         {tags.TYPE: 'builtins.object'})
 
         inflated = self.unpickler.restore(flattened)
         self.assertEqual(inflated.typeref, object)
@@ -381,6 +436,26 @@ class PicklingTestCase(unittest.TestCase):
         self.assertEqual(expect, actual)
         self.assertTrue(expect is actual)
 
+    def test_builtin_function(self):
+        expect = dir
+        json = jsonpickle.encode(expect)
+        actual = jsonpickle.decode(json)
+        self.assertEqual(expect, actual)
+        self.assertTrue(expect is actual)
+
+    def test_restore_legacy_builtins(self):
+        """
+        jsonpickle 0.9.6 and earlier used the Python 2 `__builtin__`
+        naming for builtins. Ensure those can be loaded until they're
+        no longer supported.
+        """
+        ae = jsonpickle.decode('{"py/type": "__builtin__.AssertionError"}')
+        assert ae is AssertionError
+        ae = jsonpickle.decode('{"py/type": "exceptions.AssertionError"}')
+        assert ae is AssertionError
+        cls = jsonpickle.decode('{"py/type": "__builtin__.int"}')
+        assert cls is int
+
 
 class JSONPickleTestCase(SkippableTest):
 
@@ -391,6 +466,22 @@ class JSONPickleTestCase(SkippableTest):
             % tags.OBJECT
         )
 
+    def test_API_names(self):
+        """
+        Enforce expected names in main module
+        """
+        names = list(vars(jsonpickle))
+        self.assertIn('pickler', names)
+        self.assertIn('unpickler', names)
+        self.assertIn('JSONBackend', names)
+        self.assertIn('__version__', names)
+        self.assertIn('register', names)
+        self.assertIn('unregister', names)
+        self.assertIn('Pickler', names)
+        self.assertIn('Unpickler', names)
+        self.assertIn('encode', names)
+        self.assertIn('decode', names)
+
     def test_encode(self):
         expect = self.obj
         pickle = jsonpickle.encode(self.obj)
@@ -421,7 +512,10 @@ class JSONPickleTestCase(SkippableTest):
         self.assertEqual(type(self.obj), type(actual))
 
     def test_unicode_dict_keys(self):
-        uni = unichr(0x1234)
+        if PY2:
+            uni = unichr(0x1234)  # noqa
+        else:
+            uni = chr(0x1234)
         pickle = jsonpickle.encode({uni: uni})
         actual = jsonpickle.decode(pickle)
         self.assertTrue(uni in actual)
@@ -475,7 +569,7 @@ class JSONPickleTestCase(SkippableTest):
         thing = Thing('random')
         pickle = jsonpickle.encode({thing: True})
         actual = jsonpickle.decode(pickle)
-        self.assertEqual(actual, {unicode('Thing("random")'): True})
+        self.assertEqual(actual, {'Thing("random")': True})
 
     def test_int_dict_keys_defaults(self):
         int_dict = {1000: [1, 2]}
@@ -502,6 +596,28 @@ class JSONPickleTestCase(SkippableTest):
         unpickled = jsonpickle.decode(pickled)
         self.assertTrue('name' in unpickled)
 
+    def test_dict_subclass(self):
+        obj = UserDict()
+        obj.valid = True
+        obj.s = 'string'
+        obj.d = 'd_string'
+        obj['d'] = {}
+        obj['s'] = 'test'
+        pickle = jsonpickle.encode(obj)
+        actual = jsonpickle.decode(pickle)
+
+        self.assertEqual(type(actual), UserDict)
+        self.assertTrue('d' in actual)
+        self.assertTrue('s' in actual)
+        self.assertTrue(hasattr(actual, 'd'))
+        self.assertTrue(hasattr(actual, 's'))
+        self.assertTrue(hasattr(actual, 'valid'))
+        self.assertEqual(obj['d'], actual['d'])
+        self.assertEqual(obj['s'], actual['s'])
+        self.assertEqual(obj.d, actual.d)
+        self.assertEqual(obj.s, actual.s)
+        self.assertEqual(obj.valid, actual.valid)
+
     def test_list_of_objects(self):
         """Test that objects in lists are referenced correctly"""
         a = Thing('a')
@@ -566,8 +682,6 @@ class JSONPickleTestCase(SkippableTest):
         """Test that we can call jsonpickle.load_backend()
 
         """
-        if PY32:
-            return self.skip('no simplejson for python 3.2')
         jsonpickle.load_backend('simplejson', 'dumps', 'loads', ValueError)
         self.assertTrue(True)
 
@@ -609,7 +723,7 @@ class JSONPickleTestCase(SkippableTest):
         load_backend = jsonpickle.load_backend
         self.assertFalse(load_backend('os.path', 'bad!', 'split',
                                       AttributeError))
-        self.failIf(self._backend_is_partially_loaded('os.path'))
+        self.assertFalse(self._backend_is_partially_loaded('os.path'))
 
     def test_load_backend_raises_on_bad_decode(self):
         """Test that we ignore bad decoders"""
@@ -617,14 +731,14 @@ class JSONPickleTestCase(SkippableTest):
         load_backend = jsonpickle.load_backend
         self.assertFalse(load_backend('os.path', 'join', 'bad!',
                                       AttributeError))
-        self.failIf(self._backend_is_partially_loaded('os.path'))
+        self.assertFalse(self._backend_is_partially_loaded('os.path'))
 
     def test_load_backend_handles_bad_loads_exc(self):
         """Test that we ignore bad decoder exceptions"""
 
         load_backend = jsonpickle.load_backend
         self.assertFalse(load_backend('os.path', 'join', 'split', 'bad!'))
-        self.failIf(self._backend_is_partially_loaded('os.path'))
+        self.assertFalse(self._backend_is_partially_loaded('os.path'))
 
     def test_list_item_reference(self):
         thing = Thing('parent')
@@ -678,6 +792,70 @@ class JSONPickleTestCase(SkippableTest):
         self.assertEqual(decoded.a[1][0:3], '[1,')
         self.assertEqual(decoded.a[2][0][0:3], '[1,')
 
+    def test_can_serialize_inner_classes(self):
+
+        class InnerScope(object):
+            """Private class visible to this method only"""
+            def __init__(self, name):
+                self.name = name
+
+        obj = InnerScope('test')
+        encoded = jsonpickle.encode(obj)
+        # Single class
+        decoded = jsonpickle.decode(encoded, classes=InnerScope)
+        self._test_inner_class(InnerScope, obj, decoded)
+        # List of classes
+        decoded = jsonpickle.decode(encoded, classes=[InnerScope])
+        self._test_inner_class(InnerScope, obj, decoded)
+        # Tuple of classes
+        decoded = jsonpickle.decode(encoded, classes=(InnerScope,))
+        self._test_inner_class(InnerScope, obj, decoded)
+        # Set of classes
+        decoded = jsonpickle.decode(encoded, classes={InnerScope})
+        self._test_inner_class(InnerScope, obj, decoded)
+
+    def _test_inner_class(self, InnerScope, obj, decoded):
+        self.assertTrue(isinstance(obj, InnerScope))
+        self.assertEqual(decoded.name, obj.name)
+
+    def test_can_serialize_nested_classes(self):
+        if PY2:
+            return self.skip('Serialization of nested classes requires '
+                             'Python >= 3')
+
+        middle = Outer.Middle
+        inner = Outer.Middle.Inner
+        encoded_middle = jsonpickle.encode(middle)
+        encoded_inner = jsonpickle.encode(inner)
+
+        decoded_middle = jsonpickle.decode(encoded_middle)
+        decoded_inner = jsonpickle.decode(encoded_inner)
+
+        self.assertTrue(isinstance(decoded_middle, type))
+        self.assertTrue(isinstance(decoded_inner, type))
+        self.assertEqual(decoded_middle, middle)
+        self.assertEqual(decoded_inner, inner)
+
+    def test_can_serialize_nested_class_objects(self):
+        if PY2:
+            return self.skip('Serialization of nested classes requires '
+                             'Python >= 3')
+
+        middle_obj = Outer.Middle()
+        middle_obj.attribute = 5
+        inner_obj = Outer.Middle.Inner()
+        inner_obj.attribute = 6
+        encoded_middle_obj = jsonpickle.encode(middle_obj)
+        encoded_inner_obj = jsonpickle.encode(inner_obj)
+
+        decoded_middle_obj = jsonpickle.decode(encoded_middle_obj)
+        decoded_inner_obj = jsonpickle.decode(encoded_inner_obj)
+
+        self.assertTrue(isinstance(decoded_middle_obj, Outer.Middle))
+        self.assertTrue(isinstance(decoded_inner_obj, Outer.Middle.Inner))
+        self.assertEqual(decoded_middle_obj.attribute, middle_obj.attribute)
+        self.assertEqual(decoded_inner_obj.attribute, inner_obj.attribute)
+
 
 class PicklableNamedTuple(object):
     """
@@ -817,6 +995,18 @@ class PickleProtocol2ReduceTuple(object):
                 )
 
 
+@compat.iterator
+class ReducibleIterator(object):
+    def __iter__(self):
+        return self
+
+    def __next__(self):
+        raise StopIteration()
+
+    def __reduce__(self):
+        return ReducibleIterator, ()
+
+
 def protocol_2_reduce_tuple_func(*args):
     return PickleProtocol2ReduceTupleFunc(*args)
 
@@ -1147,6 +1337,17 @@ class PicklingProtocol2TestCase(SkippableTest):
         self.assertEqual(next(decoded), '2')
         self.assertEqual(next(decoded), '3')
 
+    def test_reduce_iterable(self):
+        """
+        An reducible object should be pickleable even if
+        it is also iterable.
+        """
+        instance = ReducibleIterator()
+        self.assertTrue(util.is_iterator(instance))
+        encoded = jsonpickle.encode(instance)
+        decoded = jsonpickle.decode(encoded)
+        self.assertIsInstance(decoded, ReducibleIterator)
+
     def test_reduce_string(self):
         """
         Ensure json pickle will accept the redirection to another object when
@@ -1316,5 +1517,6 @@ def suite():
     suite.addTest(doctest.DocTestSuite(jsonpickle.unpickler))
     return suite
 
+
 if __name__ == '__main__':
     unittest.main(defaultTest='suite')
