@@ -1,18 +1,17 @@
-# -*- coding: utf-8 -*-
-
-import base64
+from __future__ import absolute_import, division, unicode_literals
+import enum
 import collections
 import decimal
 import re
-import sys
 import unittest
 import datetime
 
 import jsonpickle
+from jsonpickle import compat
 from jsonpickle import handlers
 from jsonpickle import tags
-from jsonpickle.compat import queue, set, unicode, bytes, PY2, PY_MINOR
-import jsonpickle.util as util
+from jsonpickle import util
+from jsonpickle.compat import queue, PY2
 
 from helper import SkippableTest
 
@@ -77,6 +76,10 @@ class GetstateReturnsList(object):
     def __setstate__(self, state):
         self.x, self.y = state[0], state[1]
 
+class GetstateRecursesInfintely(object):
+    def __getstate__(self):
+        return GetstateRecursesInfintely()
+
 
 class ListSubclassWithInit(list):
 
@@ -178,32 +181,52 @@ class ThingWithClassAsDefaultFactory(collections.defaultdict):
         return self.__class__()
 
 
-try:
-    import enum
-
-    class IntEnumTest(enum.IntEnum):
-        X = 1
-        Y = 2
-
-    class StringEnumTest(enum.Enum):
-        A = 'a'
-        B = 'b'
-
-    class SubEnum(enum.Enum):
-        a = 1
-        b = 2
-
-    class EnumClass(object):
-
-        def __init__(self):
-            self.enum_a = SubEnum.a
-            self.enum_b = SubEnum.b
+class IntEnumTest(enum.IntEnum):
+    X = 1
+    Y = 2
 
 
-except ImportError:
-    IntEnumTest = None
-    StringEnumTest = None
-    EnumClass = None
+class StringEnumTest(enum.Enum):
+    A = 'a'
+    B = 'b'
+
+
+class SubEnum(enum.Enum):
+    a = 1
+    b = 2
+
+
+class EnumClass(object):
+
+    def __init__(self):
+        self.enum_a = SubEnum.a
+        self.enum_b = SubEnum.b
+
+
+class MessageTypes(enum.Enum):
+    STATUS = 'STATUS',
+    CONTROL = 'CONTROL'
+
+
+class MessageStatus(enum.Enum):
+    OK = 'OK',
+    ERROR = 'ERROR'
+
+
+class MessageCommands(enum.Enum):
+    STATUS_ALL = 'STATUS_ALL'
+
+
+class Message(object):
+
+    def __init__(self, message_type, command, status=None, body=None):
+        self.message_type = MessageTypes(message_type)
+        if command:
+            self.command = MessageCommands(command)
+        if status:
+            self.status = MessageStatus(status)
+        if body:
+            self.body = body
 
 
 class ThingWithTimedeltaAttribute(object):
@@ -214,6 +237,15 @@ class ThingWithTimedeltaAttribute(object):
         return self.offset,
 
 
+class IntKeysObject(object):
+
+    def __init__(self):
+        self.data = {0: 0}
+
+    def __getstate__(self):
+        return self.__dict__
+
+
 class AdvancedObjectsTestCase(SkippableTest):
 
     def setUp(self):
@@ -307,11 +339,7 @@ class AdvancedObjectsTestCase(SkippableTest):
 
     def test_deque_roundtrip(self):
         """Make sure we can handle collections.deque"""
-        py26 = PY2 and PY_MINOR < 7
-        if py26:
-            old_deque = collections.deque([0, 1, 2])
-        else:
-            old_deque = collections.deque([0, 1, 2], maxlen=5)
+        old_deque = collections.deque([0, 1, 2], maxlen=5)
         encoded = jsonpickle.encode(old_deque)
         new_deque = jsonpickle.decode(encoded)
         self.assertNotEqual(encoded, 'nil')
@@ -321,9 +349,8 @@ class AdvancedObjectsTestCase(SkippableTest):
         self.assertEqual(new_deque[1], 1)
         self.assertEqual(old_deque[2], 2)
         self.assertEqual(new_deque[2], 2)
-        if not py26:
-            self.assertEqual(old_deque.maxlen, 5)
-            self.assertEqual(new_deque.maxlen, 5)
+        self.assertEqual(old_deque.maxlen, 5)
+        self.assertEqual(new_deque.maxlen, 5)
 
     def test_namedtuple_roundtrip(self):
         old_nt = NamedTuple(0, 1, 2)
@@ -339,9 +366,6 @@ class AdvancedObjectsTestCase(SkippableTest):
         self.assertEqual(old_nt[2], new_nt[2])
 
     def test_counter_roundtrip(self):
-        if sys.version_info < (2, 7):
-            # collections.Counter was introduced in Python 2.7
-            return
         counter = collections.Counter({1: 2})
         encoded = jsonpickle.encode(counter)
         decoded = jsonpickle.decode(encoded)
@@ -350,9 +374,6 @@ class AdvancedObjectsTestCase(SkippableTest):
         self.assertEqual(decoded.get('1'), 2)
 
     def test_counter_roundtrip_with_keys(self):
-        if sys.version_info < (2, 7):
-            # collections.Counter was introduced in Python 2.7
-            return
         counter = collections.Counter({1: 2})
         encoded = jsonpickle.encode(counter, keys=True)
         decoded = jsonpickle.decode(encoded, keys=True)
@@ -495,10 +516,10 @@ class AdvancedObjectsTestCase(SkippableTest):
         self.assertEqual(restored.data, data)
 
     def test_decimal(self):
-        obj = decimal.Decimal(1)
+        obj = decimal.Decimal('0.5')
         flattened = self.pickler.flatten(obj)
         inflated = self.unpickler.restore(flattened)
-        self.assertEqual(type(inflated), decimal.Decimal)
+        self.assertTrue(isinstance(inflated, decimal.Decimal))
 
     def test_repr_using_jsonpickle(self):
         thing = ObjWithJsonPickleRepr()
@@ -521,39 +542,31 @@ class AdvancedObjectsTestCase(SkippableTest):
         self.assertTrue('<BrokenReprThing "test">' in flattened)
         self.assertTrue(flattened['<BrokenReprThing "test">'])
 
+    def test_ordered_dict(self):
+        d = collections.OrderedDict([('c', 3), ('a', 1), ('b', 2)])
+        encoded = jsonpickle.encode(d)
+        decoded = jsonpickle.decode(encoded)
+        self.assertEqual(d, decoded)
+
+    def test_ordered_dict_unpicklable(self):
+        d = collections.OrderedDict([('c', 3), ('a', 1), ('b', 2)])
+        encoded = jsonpickle.encode(d, unpicklable=False)
+        decoded = jsonpickle.decode(encoded)
+        self.assertEqual(d, decoded)
 
     def test_ordered_dict_reduces(self):
-        if sys.version_info < (2, 7):
-            return
-
-        d = collections.OrderedDict()
-        d.update(c=3)
-        d.update(a=1)
-        d.update(b=2)
-
+        d = collections.OrderedDict([('c', 3), ('a', 1), ('b', 2)])
         has_reduce, has_reduce_ex = util.has_reduce(d)
-
         self.assertTrue(util.is_reducible(d))
         self.assertTrue(has_reduce or has_reduce_ex)
 
-        
-    def test_ordered_dict(self):
-        if sys.version_info < (2, 7):
-            return
-
-        d = collections.OrderedDict()
-        d.update(c=3)
-        d.update(a=1)
-        d.update(b=2)
-
-        encoded = jsonpickle.encode(d)
-        decoded = jsonpickle.decode(encoded)
-
-        self.assertEqual(d, decoded)
+    def test_int_keys_in_object_with_getstate_only(self):
+        obj = IntKeysObject()
+        encoded = jsonpickle.encode(obj, keys=True)
+        decoded = jsonpickle.decode(encoded, keys=True)
+        self.assertEqual(obj.data, decoded.data)
 
     def test_ordered_dict_int_keys(self):
-        if sys.version_info < (2, 7):
-            return
         d = {
             1: collections.OrderedDict([(2, -2), (3, -3)]),
             4: collections.OrderedDict([(5, -5), (6, -6)]),
@@ -569,6 +582,20 @@ class AdvancedObjectsTestCase(SkippableTest):
         self.assertEqual(-6, decoded[4][6])
         self.assertEqual(d, decoded)
 
+    def test_ordered_dict_nested(self):
+        bottom = collections.OrderedDict([('z', 1), ('a', 2)])
+        middle = collections.OrderedDict([('c', bottom)])
+        top = collections.OrderedDict([('b', middle)])
+
+        encoded = jsonpickle.encode(top)
+        decoded = jsonpickle.decode(encoded)
+        self.assertEqual(top, decoded)
+
+        # test unpicklable=False
+        encoded = jsonpickle.encode(top, unpicklable=False)
+        decoded = jsonpickle.decode(encoded)
+        self.assertEqual(top, decoded)
+
     def test_posix_stat_result(self):
         try:
             import posix
@@ -593,12 +620,16 @@ class AdvancedObjectsTestCase(SkippableTest):
         obj = DictSubclass()
         obj['key1'] = 1
 
+        expect = {
+                tags.OBJECT: 'object_test.DictSubclass',
+                'key1': 1,
+                '__dict__': {},
+        }
         flattened = self.pickler.flatten(obj)
-        self.assertEqual({'key1': 1, tags.OBJECT: 'object_test.DictSubclass'},
-                         flattened)
-        self.assertEqual(flattened[tags.OBJECT], 'object_test.DictSubclass')
+        self.assertEqual(expect, flattened)
 
         inflated = self.unpickler.restore(flattened)
+        self.assertEqual(type(inflated), DictSubclass)
         self.assertEqual(1, inflated['key1'])
         self.assertEqual(inflated.name, 'Test')
 
@@ -690,6 +721,11 @@ class AdvancedObjectsTestCase(SkippableTest):
         restored = self.unpickler.restore(flat)
         self.assertEqual(expect, restored)
 
+    def test_getstate_does_not_recurse_infinitely(self):
+        obj = GetstateRecursesInfintely()
+        pickler = jsonpickle.pickler.Pickler(max_depth=5)
+        pickler.flatten(obj)
+
     def test_thing_with_queue(self):
         obj = ThingWithQueue()
         flattened = self.pickler.flatten(obj)
@@ -727,12 +763,11 @@ class AdvancedObjectsTestCase(SkippableTest):
         self.assertEqual(type(roundtrip), object)
 
     def test_enum34(self):
-        if IntEnumTest is None or StringEnumTest is None:
-            return self.skip('enum34 module is not installed')
-
         restore = self.unpickler.restore
         flatten = self.pickler.flatten
-        roundtrip = lambda obj: restore(flatten(obj))
+
+        def roundtrip(obj):
+            return restore(flatten(obj))
         self.assertTrue(roundtrip(IntEnumTest.X) is IntEnumTest.X)
         self.assertTrue(roundtrip(IntEnumTest) is IntEnumTest)
 
@@ -740,8 +775,6 @@ class AdvancedObjectsTestCase(SkippableTest):
         self.assertTrue(roundtrip(StringEnumTest) is StringEnumTest)
 
     def test_enum34_nested(self):
-        if EnumClass is None:
-            return self.skip('enum34 module is not installed')
         ec = EnumClass()
         encoded = jsonpickle.encode(ec)
         decoded = jsonpickle.decode(encoded)
@@ -750,8 +783,6 @@ class AdvancedObjectsTestCase(SkippableTest):
         self.assertEqual(ec.enum_b, decoded.enum_b)
 
     def test_enum_references(self):
-        if IntEnumTest is None:
-            return self.skip('enum34 module is not installed')
         a = IntEnumTest.X
         b = IntEnumTest.X
 
@@ -760,45 +791,52 @@ class AdvancedObjectsTestCase(SkippableTest):
         decoded = jsonpickle.decode(encoded)
         self.assertEqual(enums_list, decoded)
 
+    def test_enum_unpicklable(self):
+        obj = Message(MessageTypes.STATUS, MessageCommands.STATUS_ALL)
+        encoded = jsonpickle.encode(obj, unpicklable=False)
+        decoded = jsonpickle.decode(encoded)
+        self.assertTrue('message_type' in decoded)
+        self.assertTrue('command' in decoded)
+
     def test_bytes_unicode(self):
         b1 = b'foo'
         b2 = b'foo\xff'
-        u1 = unicode('foo')
+        u1 = 'foo'
 
         # unicode strings get encoded/decoded as is
         encoded = self.pickler.flatten(u1)
         self.assertTrue(encoded == u1)
-        self.assertTrue(type(encoded) is unicode)
+        self.assertTrue(isinstance(encoded, compat.ustr))
         decoded = self.unpickler.restore(encoded)
         self.assertTrue(decoded == u1)
-        self.assertTrue(type(decoded) is unicode)
+        self.assertTrue(isinstance(decoded, compat.ustr))
 
         # bytestrings are wrapped in PY3 but in PY2 we try to decode first
         encoded = self.pickler.flatten(b1)
         if PY2:
             self.assertEqual(encoded, u1)
-            self.assertEqual(type(encoded), unicode)
+            self.assertTrue(isinstance(encoded, compat.ustr))
         else:
             self.assertNotEqual(encoded, u1)
-            b64ustr= base64.encodestring(b'foo').decode('utf-8')
-            self.assertEqual({tags.B64: b64ustr}, encoded)
-            self.assertEqual(type(encoded[tags.B64]), unicode)
+            encoded_ustr = util.b64encode(b'foo')
+            self.assertEqual({tags.B64: encoded_ustr}, encoded)
+            self.assertTrue(isinstance(encoded[tags.B64], compat.ustr))
         decoded = self.unpickler.restore(encoded)
         self.assertTrue(decoded == b1)
         if PY2:
-            self.assertTrue(type(decoded) is unicode)
+            self.assertTrue(isinstance(decoded, compat.ustr))
         else:
-            self.assertTrue(type(decoded) is bytes)
+            self.assertTrue(isinstance(decoded, bytes))
 
         # bytestrings that we can't decode to UTF-8 will always be wrapped
         encoded = self.pickler.flatten(b2)
         self.assertNotEqual(encoded, b2)
-        b64ustr= base64.encodestring(b'foo\xff').decode('utf-8')
-        self.assertEqual({tags.B64: b64ustr}, encoded)
-        self.assertEqual(type(encoded[tags.B64]), unicode)
+        encoded_ustr = util.b64encode(b'foo\xff')
+        self.assertEqual({tags.B64: encoded_ustr}, encoded)
+        self.assertTrue(isinstance(encoded[tags.B64], compat.ustr))
         decoded = self.unpickler.restore(encoded)
         self.assertEqual(decoded, b2)
-        self.assertTrue(type(decoded) is bytes)
+        self.assertTrue(isinstance(decoded, bytes))
 
     def test_backcompat_bytes_quoted_printable(self):
         """Test decoding bytes objects from older jsonpickle versions"""
@@ -808,11 +846,11 @@ class AdvancedObjectsTestCase(SkippableTest):
 
         # older versions of jsonpickle used a quoted-printable encoding
         expect = b1
-        actual = self.unpickler.restore({tags.BYTES: unicode('foo')})
+        actual = self.unpickler.restore({tags.BYTES: 'foo'})
         self.assertEqual(expect, actual)
 
         expect = b2
-        actual = self.unpickler.restore({tags.BYTES: unicode('foo=FF')})
+        actual = self.unpickler.restore({tags.BYTES: 'foo=FF'})
         self.assertEqual(expect, actual)
 
     def test_nested_objects(self):
@@ -828,7 +866,7 @@ class Mixin(object):
         return True
 
 
-class UnicodeMixin(unicode, Mixin):
+class UnicodeMixin(str, Mixin):
     def __add__(self, rhs):
         obj = super(UnicodeMixin, self).__add__(rhs)
         return UnicodeMixin(obj)
@@ -843,6 +881,7 @@ class UnicodeMixinHandler(handlers.BaseHandler):
     def restore(self, obj):
         return UnicodeMixin(obj['value'])
 
+
 handlers.register(UnicodeMixin, UnicodeMixinHandler)
 
 
@@ -850,8 +889,8 @@ class ExternalHandlerTestCase(unittest.TestCase):
 
     def test_unicode_mixin(self):
         obj = UnicodeMixin('test')
-        self.assertEqual(type(obj), UnicodeMixin)
-        self.assertEqual(unicode(obj), unicode('test'))
+        self.assertTrue(isinstance(obj, UnicodeMixin))
+        self.assertEqual(obj, 'test')
 
         # Encode into JSON
         content = jsonpickle.encode(obj)
@@ -860,8 +899,8 @@ class ExternalHandlerTestCase(unittest.TestCase):
         new_obj = jsonpickle.decode(content)
         new_obj += ' passed'
 
-        self.assertEqual(unicode(new_obj), unicode('test passed'))
-        self.assertEqual(type(new_obj), UnicodeMixin)
+        self.assertEqual(new_obj, 'test passed')
+        self.assertTrue(isinstance(new_obj, UnicodeMixin))
         self.assertTrue(new_obj.ok())
 
 
